/**
  ******************************************************************************
  * @file    usart.c 
  * @author  Daniel Buergin
  * @version V0.3
  * @date    11/10/2009
  * @brief   Usart Library for Cortex M3 Chip
  ******************************************************************************
  * @copy
  *
  * Based on Code from: Peter Fleury <pfleury@gmx.ch>   http://jump.to/fleury
  *
  */ 

/* Includes ------------------------------------------------------------------*/

#include "stm32f10x.h"
#include "platform_config.h"
#include "usart.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

#define UART_RX_BUFFER_MASK ( UART_RX_BUFFER_SIZE - 1)
#define UART_TX_BUFFER_MASK ( UART_TX_BUFFER_SIZE - 1)

#if ( UART_RX_BUFFER_SIZE & UART_RX_BUFFER_MASK )
#error RX buffer size is not a power of 2
#endif
#if ( UART_TX_BUFFER_SIZE & UART_TX_BUFFER_MASK )
#error TX buffer size is not a power of 2
#endif

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

USART_InitTypeDef USART_InitStructure;
GPIO_InitTypeDef  GPIO_InitStructure;

static volatile unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
static volatile unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
static volatile unsigned char UART_TxHead;
static volatile unsigned char UART_TxTail;
static volatile unsigned char UART_RxHead;
static volatile unsigned char UART_RxTail;
static volatile unsigned char UART_LastRxError;

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

void usart1_init(uint16_t baud)
{

  // RCC_APB2PeriphClockCmd(USART1_CLK, ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_USART1, ENABLE);

  /* Configure USART1 Rx as input floating */
  GPIO_InitStructure.GPIO_Pin = USART1_RxPin;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(USART1_GPIO, &GPIO_InitStructure);

  /* Configure USART1 Tx as alternate function push-pull */
  GPIO_InitStructure.GPIO_Pin = USART1_TxPin;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_Init(USART1_GPIO, &GPIO_InitStructure);


  // USART 1 konfigurieren

  USART_InitStructure.USART_BaudRate = baud;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_2;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

  // Configure the USART1
  USART_Init(USART1, &USART_InitStructure);

  // Enable the USART Transmoit interrupt: this interrupt is generated when the
  // USART1 transmit data register is empty 
  USART_ITConfig(USART1, USART_IT_TXE, ENABLE);

  // Enable the USART Receive interrupt: this interrupt is generated when the
  // USART1 receive data register is not empty
  USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);

  // Enable USART1
  USART_Cmd(USART1, ENABLE);
}

//-------------------------------------------------------------
// Interrupt Handler for USART1
//-------------------------------------------------------------

void USART1_IRQHandler(void)
{
  uint8_t tmphead;
  uint8_t tmptail;
  uint8_t lastRxError;

  //-------------------------------------------------------------
  // Receive Data
  // ToDo: Routine aus Peter Fleury Library fuer die AVR Chips
  //       uebernommen. Dort gibt der ATmega Chip einen Status
  //       zurueck. Dieser wird in die lastRxError Variable 
  //       reincodiert. Das fehlt beim Cortex
  //
  // Aktuell wird bei einem ReceiveBuffer Overflow der Interrupt
  // abgeschaltet. Ev. nicht gut ?
  //-------------------------------------------------------------

  if(USART_GetITStatus(USART1, USART_IT_RXNE) != RESET)
  {

    // Ueberbleibsel aus der ATmega Serie
    //#if defined( AT90_UART )
    //    lastRxError = (usr & (_BV(FE)|_BV(DOR)) );
    //#elif defined( ATMEGA_USART )
    //    lastRxError = (usr & (_BV(FE)|_BV(DOR)) );
    //#elif defined( ATMEGA_USART0 )
    //    lastRxError = (usr & (_BV(FE0)|_BV(DOR0)) );
    //#elif defined ( ATMEGA_UART )
    //    lastRxError = (usr & (_BV(FE)|_BV(DOR)) );
    //#endif

    // calculate buffer index

    tmphead = ( UART_RxHead + 1) & UART_RX_BUFFER_MASK;

    if ( tmphead == UART_RxTail ) {
        // error: receive buffer overflow 
        lastRxError = UART_BUFFER_OVERFLOW >> 8;
        // Disable the USART Receive interrupt
        USART_ITConfig(USART1, USART_IT_RXNE, DISABLE);
    }else{
        // store new index 
        UART_RxHead = tmphead;
        // store received data in buffer
        UART_RxBuf[tmphead] = (USART_ReceiveData(USART1) & 0x7F);;
    }
    UART_LastRxError = lastRxError;
  }

  //-------------------------------------------------------------
  // Send Data
  //-------------------------------------------------------------

  if(USART_GetITStatus(USART1, USART_IT_TXE) != RESET)
  {

    if ( UART_TxHead != UART_TxTail) {
        // calculate and store new buffer index
        tmptail = (UART_TxTail + 1) & UART_TX_BUFFER_MASK;
        UART_TxTail = tmptail;
        // get one byte from buffer and write it to UART 
        USART_SendData(USART1, UART_TxBuf[tmptail]);
    }else{
        // tx buffer empty, disable USART1 Transmit interrupt
        USART_ITConfig(USART1, USART_IT_TXE, DISABLE);
    }
  }
}

//-------------------------------------------------------------
// Function: usart_getc()
// Purpose:  return byte from ringbuffer
// Returns:  lower byte:  received byte from ringbuffer
//           higher byte: last receive error
//-------------------------------------------------------------

unsigned int usart1_getc(void)
{
    unsigned char tmptail;
    unsigned char data;

    if ( UART_RxHead == UART_RxTail ) {
        return UART_NO_DATA;
    }

    // calculate /store buffer index
    tmptail = (UART_RxTail + 1) & UART_RX_BUFFER_MASK;
    UART_RxTail = tmptail;

    // get data from receive buffer */
    data = UART_RxBuf[tmptail];

    return (UART_LastRxError << 8) + data;
}

//-------------------------------------------------------------
// Function: usart_putc()
// Purpose:  write byte to ringbuffer for transmitting via UART
// Input:    byte to be transmitted
// Returns:  none
//-------------------------------------------------------------

void usart1_Xputc(void* p, char data)
{
    unsigned char tmphead;

    tmphead  = (UART_TxHead + 1) & UART_TX_BUFFER_MASK;

    while ( tmphead == UART_TxTail ){
        ; // wait for free space in buffer
    }

    UART_TxBuf[tmphead] = (unsigned char) data;
    UART_TxHead = tmphead;

    // enable USART interrupt
    USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
}

//-------------------------------------------------------------


void usart1_putc(unsigned char data)
{
    unsigned char tmphead;

    tmphead  = (UART_TxHead + 1) & UART_TX_BUFFER_MASK;

    while ( tmphead == UART_TxTail ){
        ; // wait for free space in buffer
    }

    UART_TxBuf[tmphead] = data;
    UART_TxHead = tmphead;

    // enable USART interrupt
    USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
}

//-------------------------------------------------------------
// Function: usart_puts()
// Purpose:  transmit string to UART
// Input:    string to be transmitted
// Returns:  none
//-------------------------------------------------------------

void usart1_puts(const char *s )
{
  while (*s)
    usart1_putc(*s++);

}

//-------------------------------------------------------------
// Function: usart_flush_rxbuffer()
// Purpose:  Flushes the Inputbuffer
// Input:    none
// Returns:  none
//-------------------------------------------------------------

void usart1_flush_rxbuffer(void)
{
    UART_RxHead = 0;
    UART_RxTail = 0;
}

